/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.InconsistentConfigurationException;
import org.ow2.petals.topology.generated.Topology;

/**
 * <p>
 * The interface of the Configuration service that handles the configuration of
 * the local PEtALS container.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @author Charles Casadei - EBM WebSourcing
 * @since Petals 2.0
 */
public interface ConfigurationService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.ConfigurationService";

    static final String JAAS_PROPS_FILE = "/jaas.properties";

    static final String PROPERTY_REGISTRY_MODE = "registry.mode";

    static final String STANDALONE = "standalone";

    static final String PEER = "peer";

    static final String MASTER = "master";

    static final String SLAVE = "slave";
    
    /**
     * Get the local container configuration.
     * 
     */
    ContainerConfiguration getContainerConfiguration();

    /**
     * <p>
     * Get the local server properties (ie. content of the file '
     * <code>server.properties</code>' or equivalent), <b>including properties
     * not configured</b> (ie. set to a <code>null</code> value).
     * </p>
     * <p>
     * Note: local server properties are provided as a {@link Map} instead of
     * {@link Properties} to be able to put <code>null</code> values.
     * </p>
     * 
     * 
     */
    Map<String, String> getAllServerProperties();
    
    /**
     * <p>
     * Get the local server properties (ie. content of the file '
     * <code>server.properties</code>' or equivalent), <b>including only
     * properties configured</b> (ie. set to a non <code>null</code> value).
     * </p>
     */
    Properties getServerProperties();

    /**
     * Get the containers configuration. This only returns the container
     * configuration created at startup from local persisted data.
     * 
     * @return
     */
    public Set<ContainerConfiguration> getContainersConfiguration();

    /**
     * Get the subdomains configuration. This only returns the subdomain
     * configuration created at startup from the local persisted data.
     * 
     * @return
     */
    public Set<SubDomainConfiguration> getSubDomainsConfiguration();

    /**
     * Get the sub domain configuration the local container belongs to.
     * 
     * @return
     */
    SubDomainConfiguration getSubDomainConfiguration();

    /**
     * Get the domain configuration the local container belongs to.
     * 
     * @return
     */
    DomainConfiguration getDomainConfiguration();

    /**
     * Get the current topology. This is the topology loaded from configuration
     * files.
     * 
     * @return
     */
    Topology getTopology() throws ConfigurationException;

    /**
     * Set the running state of the local container.
     * 
     * @param state
     *            {@code ContainerConfiguration.ContainerState}
     */
    void setContainerState(ContainerConfiguration.ContainerState state);

    /**
     * Verify the given {@code containerConfiguration} against the topology
     * configuration.
     * 
     * @param newContainerConfiguration
     *            The new container configuration
     * @param newSubDomainConfiguration
     *            The new sub-domain configuration
     * @param newDomainConfiguration
     *            The new domain configuration
     * @throws InconsistentConfigurationException
     *             Inconsistency between the topology configuration and the new
     *             container configuration
     */
    void verifyContainerTopology(final ContainerConfiguration newContainerConfiguration,
            final SubDomainConfiguration newSubDomainConfiguration,
            final DomainConfiguration newDomainConfiguration)
            throws InconsistentConfigurationException;

    /**
     * Get the container configuration.
     * 
     * @param containerName
     *            the name of the container to get configuration from. If null,
     *            returns the local container configuration.
     * @return
     */
    ContainerConfiguration getContainerConfiguration(String containerName);

    /**
     * Registers a new container configuration within the local configuration.
     * 
     * @param addedContainer
     *            the container to be registered
     * @throws ConfigurationException
     */
    void addContainerConfiguration(ContainerConfiguration addedContainer)
            throws ConfigurationException;

    /**
     * Unregisters the subdomain within the local topology.
     * 
     * @param removedContainer
     * @throws ConfigurationException
     */
    void removeContainerConfiguration(ContainerConfiguration removedContainer)
            throws ConfigurationException;

    /**
     * Registers a new subdomain within the local topology.
     * 
     * @param addedSubdomain
     * @throws ConfigurationException
     */
    void addSubdomainConfiguration(SubDomainConfiguration addedSubdomain)
            throws ConfigurationException;

    /**
     * Unregister a subdomain within the local topology.
     * 
     * @param removedSubdomain
     * @throws ConfigurationException
     */
    void removeSubdomainConfiguration(String removedSubdomain) throws ConfigurationException;

    /**
     * Validate the dynamic topology
     * 
     * @throws ConfigurationException
     */
    void validateDynamicTopology() throws ConfigurationException;

    /**
     * <p>
     * Create the container configuration based on the topology and server
     * configuration properties. Basic consistency is controlled between the 2
     * configurations.
     * </p>
     * <p>
     * Required directories are created when loading the configuration:
     * <ul>
     * <li>working area directory,</li>
     * <li></li>
     * </p>
     * <p>
     * Note 1: If present into the server configuration properties, the
     * parameter {@value #PROPERTY_TOPOLOGY_URL} is ignored, the topology is
     * provided through the parameter '<code>topology</code>'.
     * <p>
     * <b>Note 2: MUST be invoked just after having starting the Configuration
     * service and before any other component starts.</b>
     * </p>
     * 
     * @param serverLocalProperties
     *            The local properties of the current container (ie: the content
     *            of 'server.properties' or equivalent). Can be
     *            <code>null</code>.
     * @param topology
     *            The topology configuration. Can not be <code>null</code>.
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    public void loadConfiguration(final Properties serverLocalProperties, final Topology topology)
            throws ConfigurationException;
}
