/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.thread;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.component.InstallationContext;
import javax.management.ObjectName;

import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * The installer thread.
 * <p>
 * This thread is used by the Installer component to have
 * each component in a separate bootstrap environment.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class BootstrapThread extends AbstractThread implements Bootstrap {

    /**
     * Management object name
     */
    private ObjectName objectName;

    /**
     * The Bootstrap
     */
    private Bootstrap jbiBootStrap;

    /**
     * The installation context attribute is required by the init method. This
     * attribute is set by the init method and used by the current thread.
     */
    private InstallationContext installContext;

    /*
     * STATICS : The different states that are used to process operations
     */

    public static final String INSTALL = "INSTALL";

    public static final String UNINSTALL = "UNINSTALL";

    public static final String INIT = "INIT";

    public static final String CLEANUP = "CLEANUP";

    public static final String EXTMBEAN = "EXTMBEAN";

    /**
     * Creates a new instance of BootstrapThread
     * 
     * @param installer
     * @param jbiBootstrap
     */
    public BootstrapThread(String installerName, Bootstrap jbiBootstrap, long taskTimeout,
            LoggingUtil logger) {
        super(taskTimeout, logger);
        this.jbiBootStrap = jbiBootstrap;
        this.setName(installerName + " - Bootstrap Thread");
    }

    /**
     * Process the requested task in the dedicated thread.
     * 
     * @param action
     *            the task to execute
     * @return true if the thread needs to continue, false if shutdown is
     *         required
     */
    @Override
    protected boolean doTask(String action) {

        boolean result = true;
        try {
            if (CLEANUP.equals(action)) {
                jbiBootStrap.cleanUp();
            } else if (INSTALL.equals(action)) {
                jbiBootStrap.onInstall();
            } else if (UNINSTALL.equals(action)) {
                jbiBootStrap.onUninstall();
            } else if (INIT.equals(action)) {
                jbiBootStrap.init(installContext);
            } else if (EXTMBEAN.equals(action)) {
                objectName = jbiBootStrap.getExtensionMBeanName();
            } else if (SHUTDOWNTHREAD.equals(action)) {
                result = false;
            } else {
                jbiException = new JBIException("Unknown action : " + action);

            }
        } catch (Throwable t) {
            t.printStackTrace(System.err);
            jbiException = new JBIException("Bootstrap failure on " + action, t);
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Bootstrap#cleanUp()
     */
    public void cleanUp() throws JBIException {
        execute(CLEANUP);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Bootstrap#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        try {
            execute(EXTMBEAN);
        } catch (JBIException e) {
            // FIXME log this exception
            e.printStackTrace();
        }
        ObjectName result = objectName;
        objectName = null;
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.Bootstrap#init(javax.jbi.component.InstallationContext
     * )
     */
    public void init(InstallationContext installContext) throws JBIException {
        this.installContext = installContext;
        execute(INIT);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Bootstrap#onInstall()
     */
    public void onInstall() throws JBIException {
        execute(INSTALL);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Bootstrap#onUninstall()
     */
    public void onUninstall() throws JBIException {
        execute(UNINSTALL);
    }
}
