/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.thread;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.management.ObjectName;

import org.ow2.petals.microkernel.api.util.LoggingUtil;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * The component life cycle thread. 
 * <p>
 * Each component life cycle have its own
 * thread to process operation that requires own ClassLoader.
 * </p>
 * <p>
 * <b>NOTE:</b> This should implement the javax.jbi.component.ComponentLifeCycle
 * interface but due to the use of Thread and the start/stop methods it is
 * impossible to do this. LET'S SEE IF FUTURE JAVA VERSION ALLOWS TO DO THINGS
 * LIKE THAT.
 * </p>
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ComponentLifeCycleThread extends AbstractThread {

    /**
     * The component
     */
    private Component component;

    /**
     * The component context
     */
    private ComponentContext context;

    /**
     * The extensions Object Name. {@code Null} if not defined.
     */
    private ObjectName objectName;

    /*
     * STATICS : The actions to process
     */

    public static final String INIT = "INIT";

    public static final String START = "START";

    public static final String STOP = "STOP";

    public static final String SHUTDOWN = "SHUTDOWN";

    public static final String MBEANNAME = "MBEANNAME";

    /**
     * Constructor.
     * 
     * @param component
     * @param tasktimeout
     * @param logger
     */
    public ComponentLifeCycleThread(Component component, long tasktimeout, LoggingUtil logger) {
        super(tasktimeout, logger);
        this.component = component;
        this.setName(component.toString() + "-life Cycle Thread");
    }

    /**
     * Excute the task.
     * 
     * @param action
     * 
     * @return true if the thread needs to continue, false if shutdown is
     *         required
     */
    @Override
    protected boolean doTask(String action) {
        boolean result = true;
        try {
            if (INIT.equals(action)) {
                getComponent().getLifeCycle().init(context);
            } else if (START.equals(action)) {
                getComponent().getLifeCycle().start();
            } else if (STOP.equals(action)) {
                getComponent().getLifeCycle().stop();
            } else if (SHUTDOWN.equals(action)) {
                getComponent().getLifeCycle().shutDown();
            } else if (MBEANNAME.equals(action)) {
                objectName = getComponent().getLifeCycle().getExtensionMBeanName();
            } else if (SHUTDOWNTHREAD.equals(action)) {
                result = false;
                ExecutionContext.getProperties().clear();
            } else {
                jbiException = new JBIException("Unknown action : " + action);
            }

        } catch (Throwable t) {
            jbiException = new JBIException("Life cycle exception on " + action, t);
        }
        return result;
    }

    /**
     * @return the component
     */
    public Component getComponent() {
        return component;
    }

    /**
     * Start life cycle step
     * 
     */
    public void doStart() throws JBIException {
        execute(START);
    }

    /**
     * Stop life cycle step
     * 
     */
    public void doStop() throws JBIException {
        execute(STOP);
    }

    /**
     * Shutdown life cycle step
     * 
     */
    public void doShutdown() throws JBIException {
        execute(SHUTDOWN);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.
     * ComponentContext)
     */
    public void doInit(ComponentContext context) throws JBIException {
        this.context = context;
        execute(INIT);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        try {
            execute(MBEANNAME);
        } catch (JBIException e) {
            // FIXME log this exception
            e.printStackTrace();
        }
        ObjectName result = objectName;
        objectName = null;
        return result;

    }
}
