/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;

import static org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.zip.ZipFile;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.ZipUtil;

/**
 * Utility class for handling packages and package URLs.
 * @author Olivier Fabre - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public class PackageHelper {

    /**
     * Load the JBI descriptor from the repository folder of the given entity
     * (component, shared libraries or service assembly).
     * 
     * @param entityName
     *            name of entity
     * @param entityType
     *            type of entity (component, shared libraries or service
     *            assembly)
     * @return the {@link Jbi} object
     * @throws PetalsException
     * @throws PetalsException
     * @throws IOException
     * @throws JBIDescriptorException
     * @throws MalformedURLException
     */
    public static final Jbi loadDescriptor(String entityName, JbiArchiveType entityType,
            RepositoryService repositoryService) throws PetalsException, MalformedURLException,
            JBIDescriptorException, IOException {
        Jbi result = null;
        File tmpJBIFile = null;
        InputStream tmpJBIFileAsInputStream = null;

        try {
            switch (entityType) {
                case COMPONENT_TYPE:
                    tmpJBIFile = new File(repositoryService.getComponentInstallDirectory(entityName),
                            JBI_DESCRIPTOR_RESOURCE);
                    break;
                case SA_TYPE:
                    tmpJBIFile = new File(repositoryService.getServiceAssemblyInstallDirectory(entityName), JBI_DESCRIPTOR_RESOURCE);
                    break;
                case SL_TYPE:
                    tmpJBIFile = new File(repositoryService.getSharedLibraryInstallDirectory(entityName), JBI_DESCRIPTOR_RESOURCE);
                    break;
            }

            if (!tmpJBIFile.exists()) {
                throw new PetalsException(
                        "JBI descriptor entry was not found in repository for JBI artefact '" + entityName + "'");
            }

            tmpJBIFileAsInputStream = new FileInputStream(tmpJBIFile);
            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);

        } finally {
            if (tmpJBIFileAsInputStream != null) {
                tmpJBIFileAsInputStream.close();
            }
        }

        return result;
    }

    /**
     * Load the JBI descriptor from the given archive file
     * 
     * @param archiveURI
     *            the URI of the archive file
     * @return the {@link Jbi} object
     * @throws IOException
     * @throws JBIDescriptorException
     * @throws MalformedURLException
     * @throws PetalsException
     */
    public static final Jbi loadDescriptor(final URL archiveURL) throws MalformedURLException,
            JBIDescriptorException, IOException, PetalsException {

        try {
            final ZipFile zipArchive = new ZipFile(new File(archiveURL.toURI()));
            try {
                final File tmpJBIFile = ZipUtil.getEntryAsTemp(zipArchive, JBI_DESCRIPTOR_RESOURCE);
                try {
                    final InputStream tmpJBIFileAsInputStream = tmpJBIFile.toURI().toURL().openStream();
                    try {
                        return JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);
                    } finally {
                        tmpJBIFileAsInputStream.close();
                    }
                } finally {
                    tmpJBIFile.delete();
                }
            } finally {
                zipArchive.close();
            }
        } catch (final URISyntaxException e) {
            // A PetalsException is thrown to avoid to break the API
            throw new PetalsException(e);
        }
    }

    /**
     * Load the JBI descriptor from the unzipped package folder of the given
     * entity (component, shared lib or service assembly).
     * 
     * @param unzipRoot
     *            unzipped package root in work directory
     * @return the {@link Jbi} object
     * @throws PetalsException
     * @throws PetalsException
     * @throws JBIDescriptorException
     * @throws FileNotFoundException
     */
    public static final Jbi loadDescriptor(File packageFile) throws PetalsException,
            FileNotFoundException, JBIDescriptorException {
        Jbi result = null;
        File tmpJBIFile = null;
        InputStream tmpJBIFileAsInputStream = null;

        try {
            tmpJBIFile = new File(packageFile, JBI_DESCRIPTOR_RESOURCE);

            if (!tmpJBIFile.exists()) {
                throw new PetalsException("JBI descriptor not found in the package '" + packageFile.getAbsolutePath() + "'");
            }

            tmpJBIFileAsInputStream = new FileInputStream(tmpJBIFile);

            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);
        } finally {
            try {
                if (tmpJBIFileAsInputStream != null) {
                    tmpJBIFileAsInputStream.close();
                }
            } catch (IOException ex) {
                //Eat the exception (to avoid impacting everyone's code)...bad practice anyway
            }
        }

        return result;
    }

}
