/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;

import java.util.ArrayList;
import java.util.List;

import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * The TaskProcessor executes an ordered List of Tasks : it calls the
 * <i>execute</i> method on each Task. It ensures the system state recovery in
 * case of failure during the Tasks processing : it will call the <i>undo</i>
 * method on each allready executed Task before the whole process crash.
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public class TaskProcessor {

    protected List<Task> taskList;

    protected Context context;

    protected LoggingUtil log;

    public TaskProcessor(Context context, LoggingUtil log) {
        super();
        this.taskList = new ArrayList<Task>();
        this.context = context;
        this.log = log;
    }

    /**
     * Add a new Task in the List of Tasks to execute. Task will be processed in
     * the same order as they have been added to the List.
     * 
     * @param task
     *            a new Task to process
     */
    public void addTask(Task task) {
        this.taskList.add(task);
    }

    /**
     * Process each Task previously added to the Processor, in the same order as
     * they have been added to it. If an exception occurs during the processing
     * of a Task, the processor will undo this Task and all previously executed
     * Tasks, in the reverse order as they have been executed.
     * 
     * @return The cause of the error if the process has failed, {@code null}
     *         otherwise
     * 
     */
    public Throwable process() {
        Throwable result = null;
        for (int i = 0; (i < this.taskList.size()) && (result == null); i++) {
            try {
                this.taskList.get(i).execute(this.context);
            } catch (final Throwable e) {
                result = e;
                this.unprocess(i - 1);
            }
        }
        return result;
    }

    /**
     * Unprocess each Task previously added to the Processor from the step
     * provided, in the reversed order they have been added to it. If an error
     * occurs during a Task unprocessing, the processor skips it and continues
     * to unprocess other Tasks.
     * 
     * @param step
     *            . The task number reference into the list to start the reverse
     *            processing
     * @return {@code true} if the process is successful, {@code false}
     *         otherwise
     */
    public boolean unprocess(int step) {
        boolean result = true;
        for (int i = step; i >= 0; i--) {
            try {
                this.taskList.get(i).undo(this.context);
            } catch (Throwable e) {
                result = false;
                this.log.error("Error during task unprocessing", e);
            }
        }
        return result;
    }

    /**
     * Unprocess each Task previously added to the Processor, in the reversed
     * order they have been added to it. If an error occurs during a Task
     * unprocessing, the processor skips it and continues to unprocess other
     * Tasks.
     * 
     * @return {@code true} if the process is successful, {@code false}
     *         otherwise
     */
    public boolean unprocess() {
        return this.unprocess(this.taskList.size() - 1);
    }
}
