/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging.registry;

import java.util.List;
import java.util.Map;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

/**
 * EndpointRegistryMBean interface exposes method to retrieve information about
 * endpoints activated in the JBI environment domain.
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public interface EndpointRegistryMBean {

    /**
     * Retrieve all activated internal endpoints information for the current
     * Domain. An endpoint is mapped to a {@link Map} with the following fields
     * :
     * <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * </TABLE>
     * 
     * 
     * @return a {@link List} of all activated internal endpoints information
     *         for the current Joram Domain. Must be non null, may be empty
     */
    public List<Map<String, Object>> getAllInternalEndpoints();

    /**
     * Retrieve all activated external endpoints information for the current
     * Domain. An endpoint is mapped to a {@link Map} with the following fields
     * :
     * <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * </TABLE>
     * 
     * 
     * @return a {@link List} of all activated external endpoints information
     *         for the current Domain. Must be non null, may be empty
     */
    public List<Map<String, Object>> getAllExternalEndpoints();

    /**
     * Retrieve all activated endpoints (internal, external and connections)
     * information for the current Domain. An endpoint is mapped to a
     * {@link Map} with the following fields :
     * <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * <TR>
     * <TD>type</TD>
     * <TD>String</TD>
     * </TR>
     * </TABLE>
     * 
     * The type field follow this pattern : 'internal' if it's an internal
     * endpoint, 'external' if it's an external endpoint or
     * '{serviceNamespace}serviceLocalpart@endpointName' if it's a connection
     * 
     * @return a {@link List} of all activated endpoints (internal, external and
     *         connections) information for the current Joram Domain. Must be
     *         non null, may be empty
     */
    public List<Map<String, Object>> getAllEndpoints();

    /**
     * Returns an InputStream representation of a ServiceEndpoint description
     * 
     * @param serviceName
     *            the service {@link QName} of this endpoint
     * @param endpointName
     *            the unique name of this endpoint
     * @return a Dom representation of a ServiceEndpoint description
     * @throws JBIException
     *             if an error occurs during endpoint description retrieval
     */
    public String getDescription(String serviceName, String endpointName) throws JBIException;
    
    
    /**
     * Synchronize the registry for the local node
     *  
     * @throws Exception 
     */
    public void synchronizeRegistry() throws Exception;
}
