/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.system.repository;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;

import org.ow2.petals.microkernel.api.system.repository.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.ServiceAssemblyAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.SharedLibraryAlreadyExistsException;

/**
 * Platform Component Repository. 
 * <p>
 * This repository stores all the JBI artifacts in exploded format.
 * </p>
 * 
 * @since Petals 1.0
 * @author Olivier Fabre - EBM WebSourcing
 */
public interface RepositoryService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.System.RepositoryService";

    /**
     * Path to repository location within Petals installation directory.
     */
    static final String DEFAULT_REPOSITORY_PATH = "repository";

    /**
     * Install a component package into the local Petals repository.
     * 
     * @param componentId
     *            The component name (as JBI identifier, unique into the local
     *            repository. Must be non null and non empty.
     * @param explodedArchiveRootDirectory
     *            The root directory of teh exploded component archive. Must be
     *            non null.
     * @return the component installation root {@link File}
     * @throws IOException
     *             if it fails to create install and work dirs for the component
     *             or if it fails to copy package into install dir
     * @throws ComponentAlreadyExistsException
     *             The component already exists in the repository.
     */
    File addComponent(String componentId, File explodedArchiveRootDirectory) throws IOException,
            ComponentAlreadyExistsException;

    /**
     * Copy a service assembly package into petals repository
     * 
     * @param serviceAssemblyId
     *            <code>String</code> unique sa id that identifies an unique sa
     *            directory into the petals repository. Must be non null and non
     *            empty
     * @param tempFile
     *            <code>File</code> the service assembly package file to add
     *            into the repository. Must be non null
     * @return the sa installation root {@link File}
     * @throws IOException
     *             if it fails to create install and work dirs for the sa or if
     *             it fails to copy package into install dir
     * @throws ServiceAssemblyAlreadyExistsException
     *             The serviced assembly already exists in the repository.
     */
    File addServiceAssembly(String serviceAssemblyId, File tempFile) throws IOException,
            ServiceAssemblyAlreadyExistsException;

    /**
     * Copy a shared library package into petals repository
     * 
     * @param sharedLibraryId
     *            <code>String</code> unique sl id that identifies an unique sl
     *            directory into the petals repository. Must be non null and non
     *            empty
     * @param version
     *            Version of the shared library as defined by JBI.
     * @param tempFile
     *            <code>File</code> the shared lib package file to add into the
     *            repository. Must be non null
     * @return the sl installation root {@link File}
     * @throws IOException
     *             if it fails to create install and work dirs for the sl or if
     *             it fails to copy package into install dir
     * @throws SharedLibraryAlreadyExistsException
     *             The sl already exists in the repository.
     */
    File addSharedLibrary(String sharedLibraryId, String version, File tempFile)
            throws IOException, SharedLibraryAlreadyExistsException;

    /**
     * Explode a given service unit archive into a SU install root created into
     * the SA install root
     * 
     * @param suId
     *            <code>String</code> unique SU identifier within the
     *            repository.
     * @param suZipLocation
     *            <code>URL</code> the SU Zip archive file to add flatten into
     *            the repository.
     * @param saId
     *            <code>String</code> unique SA identifier within the repository
     *            where SU add to be flatten
     * @return <code>File</code> installation root of su in the repository. Must
     *         be a valid directory.
     * @throws IOException
     *             if SU install root can't be created or SU archive can't be
     *             unzipped
     */
    File explodeSUIntoSAInstallDirectory(String serviceUnitId, URL suZipLocation,
            String serviceAssemblyId) throws IOException;

    /**
     * Returns the root directory of this component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the root directory of this component, null if it doesn't exist
     * @throws IOException
     *             if an error occurred during component root retrieval
     */
    File getComponentDirectory(String componentId) throws IOException;

    /**
     * Returns the install directory of the component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the install directory of the component
     * @throws IOException
     *             if an error occurred during component root retrieval
     */
    File getComponentInstallDirectory(String componentId) throws IOException;

    /**
     * Retrieve the components directory absolute path from the Petals
     * repository. This directory contains all installed components.
     * 
     * @return the components directory URL from the Petals repository
     */
    File getComponentsDirectory();

    /**
     * Returns the work directory of the component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the work directory of the component
     * @throws IOException
     *             if an error occurred during component root retrieval
     */
    File getComponentWorkDirectory(String componentId) throws IOException;

    /**
     * Retrieve the "lost+found" directory absolute path from the Petals
     * repository. This directory contains all unknown SharedLibraries,
     * Components and ServiceAssemblies.
     * 
     * @return the lost+found directory
     */
    File getLostPlusFoundDirectory();

    /**
     * Return petals repository folder. The repository folder contains extracted
     * component, service assembly and shared library packages. It will be
     * created when this method is called for the first time
     * 
     * @return PEtALS repository directory
     */
    File getRepositoryDirectory();

    /**
     * Retrieve the ServiceAssemblies directory absolute path from the Petals
     * repository. This directory contains all deployed ServiceAssemblies.
     * 
     * @return the ServiceAssemblies directory
     */
    File getServiceAssembliesDirectory();

    /**
     * Returns the root directory of the service assembly
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique sa
     *            directory into the petals repository.
     * @return the root directory of this sa, null if it doesn't exist
     * @throws IOException
     *             if an error occured during service assembly root retrieval
     */
    File getServiceAssemblyDirectory(String serviceAsemblyId) throws IOException;

    /**
     * Returns the install directory of the service assembly
     * 
     * @param serviceAssemblyId
     *            <code>String</code> unique sa id that identifies an unique sa
     *            directory into the petals repository.
     * @return the install directory of the sa
     * @throws IOException
     *             if an error occured during service assembly root retrieval
     */
    File getServiceAssemblyInstallDirectory(String serviceAssemblyId) throws IOException;

    /**
     * Returns the work directory of the service assembly
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique sa
     *            directory into the petals repository.
     * @return the work directory of the sa
     * @throws IOException
     *             if an error occured during service assembly root retrieval
     */
    File getServiceAssemblyWorkDirectory(String serviceAsemblyId) throws IOException;

    /**
     * Retrieve the SharedLibraries directory absolute path from the Petals
     * repository. This directory contains all installed SharedLibraries.
     * 
     * @return the SharedLibs directory URL from the Petals repository
     */
    File getSharedLibrariesDirectory();

    /**
     * Returns the root directory of the shared library
     * 
     * @param sharedLibraryId
     *            <code>String</code> unique sl id that identifies an unique sl
     *            directory into the petals repository.
     * @return the root directory of this sl, null if it doesn't exist
     * @throws IOException
     *             if an error occured during shared lib root retrieval
     */
    File getSharedLibraryDirectory(String sharedLibraryId) throws IOException;

    /**
     * Returns the install directory of the shared library
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique sl
     *            directory into the petals repository.
     * @return the install directory of the sl
     * @throws IOException
     *             if an error occured during shared lib root retrieval
     */
    File getSharedLibraryInstallDirectory(String sharedLibraryId) throws IOException;

    /**
     * Returns the work directory of the shared library
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique sl
     *            directory into the petals repository.
     * @return the work directory of the sl
     * @throws IOException
     *             if an error occured during shared lib root retrieval
     */
    File getSharedLibraryWorkDirectory(String sharedLibraryId) throws IOException;

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given component identifier.
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @throws FileNotFoundException
     *             the component does not exist in the repository.
     * @throws IOException
     *             if an error occurred during the delete of related files
     */
    void removeComponent(String componentId) throws FileNotFoundException, IOException;

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given sa identifier.
     * 
     * @param serviceAssemblyId
     *            <code>String</code> unique sa id that identifies an unique sa
     *            directory into the petals repository.
     * @throws FileNotFoundException
     *             the service assembly does not exist in the repository.
     * @throws IOException
     *             if an error occurred during the delete of related files
     */
    void removeServiceAssembly(String serviceAssemblyId) throws FileNotFoundException, IOException;

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given sl identifier.
     * 
     * @param sharedLibraryId
     *            unique sl id that identifies an unique sl directory into the
     *            petals repository.
     * @param version
     *            Version of the shared library as defined by JBI.
     * @throws FileNotFoundException
     *             the shared library does not exist in the repository.
     * @throws IOException
     *             if an error occurred during the delete of related files
     */
    void removeSharedLibrary(String sharedLibraryId, String version) throws FileNotFoundException,
            IOException;
}
