/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.SyncFailedException;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.channels.OverlappingFileLockException;
import java.nio.channels.ReadableByteChannel;

/**
 * @author Rafael Marins - Fossil E-Commerce
 */
public final class NioUtil {

    /**
     * Size of buffer in bytes used during some operations.
     */
    private static final int BUFFER_SIZE = 4096;

    /**
     * Do not construct.
     */
    private NioUtil() {
        super();
    }

    /**
     * Transfer bytes blocks using direct buffer memory allocation. Relies in
     * use the faster Java New I/O instead of traditional byte array move.
     * 
     * @param is
     *            the {@link InputStream} to copy
     * @param aFile
     *            the output {@link File}
     * @throws IOException
     *             if errors occured
     */
    public static final void copyStreamToFile(InputStream is, File aFile) throws IOException {
        ReadableByteChannel inputChannel = Channels.newChannel(is);
        FileOutputStream fos = new FileOutputStream(aFile);
        FileChannel outputChannel = fos.getChannel();
        try {
            ByteBuffer buffer = ByteBuffer.allocate(BUFFER_SIZE);
            int readBytes;
            while ((readBytes = inputChannel.read(buffer)) != -1) {
                if (readBytes > 0) {
                    buffer.flip();
                    outputChannel.write(buffer);
                    buffer.clear();
                }
            }
        } finally {
            try {
                if (inputChannel != null) {
                    inputChannel.close();
                }
            } catch (IOException e) {
                // do nothing
            }
            try {
                if (outputChannel != null) {
                    outputChannel.close();
                }
            } catch (IOException e) {
                // do nothing
            }
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                // do nothing
            }

        }
    }

    /**
     * check if the file can be read
     * 
     * @param file
     * @return true if the file is not locked, false otherwise
     */
    public static final boolean checkAccess(File file) {
        boolean result = false;
        // try to access the file to test if it can be read.
        // We use only the mode 'r' to be able to process read-only files.
        try {
            final String fileMode;
            if (file.canWrite()) {
                fileMode = "rw";
            } else {
                fileMode = "r";
            }

            final RandomAccessFile randomAccessFile = new RandomAccessFile(file, fileMode);
            randomAccessFile.getFD().sync();
            final FileChannel channel = randomAccessFile.getChannel();
            
            try {
                // Use the file channel to create a lock on the file.
                // This method blocks until it can retrieve the lock.
                final FileLock lock;
                if (file.canWrite()) {
                    // The file is in read-write mode, we try to set an
                    // exclusive lock.
                    lock = channel.lock();
                } else {
                    // The file is in read-only mode, we try to set a shared
                    // lock.
                    lock = channel.lock(0L, Long.MAX_VALUE, true);
                }
                lock.release();
                result = true;
            } catch (OverlappingFileLockException e) {
            } catch (IOException e) {
                // do nothing
            } finally {
                try {
                    channel.close();
                } catch (final IOException e) {
                    // do nothing
                }
            }
        } catch (final FileNotFoundException e) {
            // do nothing
        } catch (final SyncFailedException e) {
            // do nothing
        } catch (final IOException e) {
            // do nothing
        }
        return result;
    }

}
