/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.io.IOUtils;
import org.ow2.petals.basisapi.exception.PetalsException;

/**
 * Contains utilities methods in relation to Zip operations.
 * @author Rafael Marins - Fossil E-Commerce
 * @author ddesjardins - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public final class ZipUtil {

    /**
     * Do not construct.
     */
    private ZipUtil() {
        super();
    }

    /**
     * Create a temporary File from a given entry of a given ZipFile. You must
     * delete the temp file after using it.
     * 
     * @param zipFile
     *            the Zipped file to search in, must be non null
     * @param entryName
     *            the name of an entry in the Zip file, must be non null and non
     *            empty
     * @throws PetalsException
     *             if entry can't be found or if an error occured during zip
     *             reading
     * @throws IOException
     * @return a temp File read from the Zip entry, not <code>null</code>.
     */
    public static File getEntryAsTemp(ZipFile zipFile, String entryName) throws PetalsException,
            IOException {

        File tempFile = null;
        InputStream zipStream = null;
        ZipEntry theEntry = getZipEntry(zipFile, entryName);
        zipStream = zipFile.getInputStream(theEntry);
        tempFile = File.createTempFile("petals-", null);
        NioUtil.copyStreamToFile(zipStream, tempFile);

        return tempFile;
    }

    private static ZipEntry getZipEntry(ZipFile zipFile, String entryName) throws PetalsException {
        ZipEntry theEntry = null;
        if (entryName.indexOf("/") > -1) {
            theEntry = zipFile.getEntry(entryName);
            if (theEntry == null) {
                theEntry = zipFile.getEntry(entryName.replace("/", "\\"));
                if (theEntry == null) {
                    throw new PetalsException("Zip Entry not found: " + entryName);
                }
            }
        } else {
            theEntry = zipFile.getEntry(entryName);
            if (theEntry == null) {
                theEntry = zipFile.getEntry(entryName.replace("\\", "/"));
                if (theEntry == null) {
                    throw new PetalsException("Zip Entry not found: " + entryName);
                }
            }
        }
        return theEntry;
    }

    private static void unzipEntry(ZipFile zipFile, ZipEntry entry, File destDir)
            throws IOException {
        InputStream in = null;
        OutputStream out = null;
        try {
            // Open the compressed entry
            in = zipFile.getInputStream(entry);

            // Open the output file
            out = new FileOutputStream(destDir);

            // Transfer bytes from the compressed file to the output file
            IOUtils.copy(in, out);

        } finally {
            // Close the file and stream
            if (in != null) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
        }
    }

    /**
     * Unzip given zip file into the given destination
     * 
     * @param zipFile
     *            the File to unzip, cannot be null
     * @param destDir
     *            the destination directory, can be null
     * @throws IOException
     *             if an I/O error occurred
     */
    public static void unzipArchive(File zipFile, File destDir) throws IOException {
        final ZipFile zip = new ZipFile(zipFile);

        try {
            Enumeration<? extends ZipEntry> en = zip.entries();
            while (en.hasMoreElements()) {
                ZipEntry entry = (ZipEntry) en.nextElement();
                File file = null;
                if (destDir != null) {
                    file = new File(destDir, entry.getName());
                } else {
                    file = new File(entry.getName());
                }

                if (entry.isDirectory()) {
                    if (!file.mkdirs()) {
                        throw new IOException("Error creating directory: " + file);
                    }
                } else {
                    File parent = file.getParentFile();
                    if (parent != null && !parent.exists()) {
                        if (!parent.mkdirs()) {
                            throw new IOException("Error creating directory: " + parent);
                        }
                    }
                    unzipEntry(zip, entry, file);
                }
            }
        } finally {
            zip.close();
        }
    }
}
