/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.zeroturnaround.zip.ZipUtil;

/**
 * Unit tests of {@link PackageHelper}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class PackageHelperTest {

    private static final String EXPLODED_JBI_ARCHIVE = "exploded-component";

    private static final String COMPONENT_NAME = "identifier";

    @Rule
    public final TemporaryFolder temporaryFolder = new TemporaryFolder();

    /**
     * <p>
     * Load, from a file, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located in a valid directory.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromFile() throws FileNotFoundException, PetalsException, JBIDescriptorException,
            URISyntaxException {

        final URL packageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", packageURL);

        final Jbi jbi = PackageHelper.loadDescriptor(new File(packageURL.toURI()));

        assertJbi(jbi);

    }

    /**
     * <p>
     * Load, from a file, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located in a valid directory containing a space.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromFile_DirNameContainsSapce() throws PetalsException, JBIDescriptorException,
            URISyntaxException, IOException {

        final File targetPackageDir = this.temporaryFolder.newFolder("dir with space");

        final URL initialPackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", initialPackageURL);

        FileUtils.copyDirectory(new File(initialPackageURL.toURI()), targetPackageDir);

        final Jbi jbi = PackageHelper.loadDescriptor(targetPackageDir);

        assertJbi(jbi);
    }

    /**
     * <p>
     * Load, from an URL, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located into an URL pointing to a valid directory.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromURL() throws MalformedURLException, JBIDescriptorException, IOException,
            PetalsException, URISyntaxException {

        final URL explodedpackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", explodedpackageURL);

        final File packageFile = this.temporaryFolder.newFile("component.zip");
        ZipUtil.pack(new File(explodedpackageURL.toURI()), packageFile);

        final Jbi jbi = PackageHelper.loadDescriptor(packageFile.toURI().toURL());

        assertJbi(jbi);

    }

    /**
     * <p>
     * Load, from an URL, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located into an URL pointing to a valid directory containing a space.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromURL_DirNameContainsSapce() throws PetalsException, JBIDescriptorException,
            URISyntaxException, IOException {

        final File targetPackageDir = this.temporaryFolder.newFolder("dir with space");

        final URL explodedpackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", explodedpackageURL);

        final File packageFile = this.temporaryFolder.newFile("component.zip");
        ZipUtil.pack(new File(explodedpackageURL.toURI()), packageFile);

        FileUtils.copyFileToDirectory(packageFile, targetPackageDir);

        final File targetPackageFile = new File(targetPackageDir, packageFile.getName());
        final Jbi jbi = PackageHelper.loadDescriptor(targetPackageFile.toURI().toURL());

        assertJbi(jbi);
    }

    private final void assertJbi(final Jbi jbi) {

        assertNotNull(jbi);
        assertNotNull(jbi.getComponent());
        assertNotNull(jbi.getComponent().getIdentification());
        assertEquals(COMPONENT_NAME, jbi.getComponent().getIdentification().getName());
    }

}
